'---------------------------------------------------------------------
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
'KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'PARTICULAR PURPOSE.
'---------------------------------------------------------------------

Option Explicit On
Option Strict On

Imports System.Collections
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Diagnostics
Imports System.Drawing
Imports System.Drawing.Design
Imports System.Windows.Forms
Imports System.Windows.Forms.Design
Imports System.Windows.Forms.Design.Behavior

Namespace Extensions

    ''' <summary>
    ''' An <see cref="DesignSurfaceExtension"/> for managing 
    ''' the <see cref="Control.Anchor"/> property of all controls
    ''' on the design surface.
    ''' </summary>
    Friend Class AnchorExtension
        Inherits DesignSurfaceExtension

        Friend Sub New(ByVal serviceProvider As IServiceProvider)
            MyBase.New(serviceProvider)

        End Sub

        ''' <summary>
        ''' Creates four <see cref="AnchorGlyph"/> instances and 
        ''' adds them to the given adorner's <see cref="Adorner.Glyphs"/> collection. 
        ''' </summary>
        ''' <param name="c">The <see cref="Control"/> which is related to 
        ''' the new <see cref="AnchorGlyph"/> instances.</param>

        Protected Overrides Sub AddGlyphsForControl(ByVal c As Control)
            If c IsNot Nothing Then
                If Not (TypeOf c Is Form) Then
                    Dim a As Adorner = Me.adornerValue

                    a.Glyphs.Add(New AnchorGlyph( _
                        c, _
                        AnchorStyles.Left, _
                        MyBase.BehaviorService, _
                        MyBase.ComponentChangeService, _
                        MyBase.SelectionService, _
                        MyBase.ReferenceService, _
                        MyBase.ServiceProvider, _
                        a))

                    a.Glyphs.Add(New AnchorGlyph( _
                        c, _
                        AnchorStyles.Top, _
                        MyBase.BehaviorService, _
                        MyBase.ComponentChangeService, _
                        MyBase.SelectionService, _
                        MyBase.ReferenceService, _
                        MyBase.ServiceProvider, _
                        a))

                    a.Glyphs.Add(New AnchorGlyph( _
                        c, _
                        AnchorStyles.Right, _
                        MyBase.BehaviorService, _
                        MyBase.ComponentChangeService, _
                        MyBase.SelectionService, _
                        MyBase.ReferenceService, _
                        MyBase.ServiceProvider, _
                        a))

                    a.Glyphs.Add(New AnchorGlyph( _
                        c, _
                        AnchorStyles.Bottom, _
                        MyBase.BehaviorService, _
                        MyBase.ComponentChangeService, _
                        MyBase.SelectionService, _
                        MyBase.ReferenceService, _
                        MyBase.ServiceProvider, _
                        a))
                End If
            End If
        End Sub

        ''' <summary>
        ''' Removes the <see cref="AnchorGlyph"/> instances associated with
        ''' the given control from the <see cref="Adorner.Glyphs"/> collection.
        ''' </summary>
        ''' <param name="c">The <see cref="Control"/> associated with 
        ''' the <see cref="AnchorGlyph"/> instances.</param>
        Protected Overrides Sub RemoveGlyphsForControl(ByVal c As Control)
            If c IsNot Nothing Then
                For i As Integer = Me.adornerValue.Glyphs.Count - 1 To 0 Step -1
                    Dim g As AnchorGlyph = CType(Me.adornerValue.Glyphs(i), AnchorGlyph)

                    If Object.ReferenceEquals(c, g.RelatedControl) Then
                        Me.adornerValue.Glyphs.Remove(g)
                    End If
                Next
            End If
        End Sub

        Public Overrides ReadOnly Property ActionList() As DesignerActionList
            Get
                If Me.actionListValue Is Nothing Then
                    Me.actionListValue = New AnchorActionList(Me)
                End If
                Return Me.actionListValue
            End Get
        End Property

    End Class

End Namespace
