'---------------------------------------------------------------------
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
'KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'PARTICULAR PURPOSE.
'---------------------------------------------------------------------

Option Explicit On
Option Strict On

Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms

Namespace Extensions

    Friend Class MarginBehavior
        Inherits BoundsBehavior

        Friend Sub New(ByVal serviceProvider As IServiceProvider)
            MyBase.New(serviceProvider)

            Me.propertyName = "Margin"
            Me.transactionName = "Margin drag"
        End Sub

        Protected Overrides Sub SetBoundsProperty(ByVal pd As PropertyDescriptor)
            If pd IsNot Nothing Then
                Dim translatedBounds As Rectangle = New Rectangle( _
                    Me.behaviorServiceValue.ControlToAdornerWindow(Me.relatedControl), _
                    Me.relatedControl.Size)

                Select Case (Me.selectedEdge)
                    Case EdgeSelection.Left
                        Dim leftMargin As Integer = translatedBounds.Left - Me.currentPosition.X

                        If leftMargin >= 0 Then
                            Dim margin As Padding = New Padding( _
                                leftMargin, _
                                Me.relatedControl.Margin.Top, _
                                Me.relatedControl.Margin.Right, _
                                Me.relatedControl.Margin.Bottom)

                            pd.SetValue(Me.relatedControl, margin)
                        End If

                    Case EdgeSelection.Top
                        Dim topMargin As Integer = translatedBounds.Top - Me.currentPosition.Y

                        If topMargin >= 0 Then
                            Dim margin As Padding = New Padding( _
                                Me.relatedControl.Margin.Left, _
                                topMargin, _
                                Me.relatedControl.Margin.Right, _
                                Me.relatedControl.Margin.Bottom)

                            pd.SetValue(Me.relatedControl, margin)
                        End If

                    Case EdgeSelection.Right
                        Dim rightMargin As Integer = Me.currentPosition.X - translatedBounds.Right

                        If rightMargin >= 0 Then
                            Dim margin As Padding = New Padding( _
                                Me.relatedControl.Margin.Left, _
                                Me.relatedControl.Margin.Top, _
                                rightMargin, _
                                Me.relatedControl.Margin.Bottom)

                            pd.SetValue(Me.relatedControl, margin)
                        End If

                    Case EdgeSelection.Bottom
                        Dim bottomMargin As Integer = Me.currentPosition.Y - translatedBounds.Bottom

                        If bottomMargin >= 0 Then
                            Dim margin As Padding = New Padding( _
                                Me.relatedControl.Margin.Left, _
                                Me.relatedControl.Margin.Top, _
                                Me.relatedControl.Margin.Right, _
                                bottomMargin)

                            pd.SetValue(Me.relatedControl, margin)
                        End If

                    Case EdgeSelection.None

                    Case Else

                End Select
            End If
        End Sub

    End Class

End Namespace
