'---------------------------------------------------------------------
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
'KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'PARTICULAR PURPOSE.
'---------------------------------------------------------------------

Option Explicit On
Option Strict On

Imports System.Collections
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Diagnostics
Imports System.Drawing
Imports System.Drawing.Design
Imports System.Windows.Forms
Imports System.Windows.Forms.Design
Imports System.Windows.Forms.Design.Behavior

Namespace Extensions

    ''' <summary>
    ''' An <see cref="DesignSurfaceExtension"/> for managing 
    ''' the <see cref="Control.Padding"/> property of all controls
    ''' on the design surface.
    ''' </summary>
    Friend Class PaddingExtension
        Inherits DesignSurfaceExtension

        Friend Sub New(ByVal serviceProvider As IServiceProvider)
            MyBase.New(serviceProvider)
        End Sub

        ''' <summary>
        ''' Creates a <see cref="PaddingGlyph"/> and adds it to the given
        ''' adorner's <see cref="Adorner.Glyphs"/> collection.
        ''' </summary>
        ''' <param name="c">The <see cref="Control"/> which is related to 
        ''' the new <see cref="PaddingGlyph"/>.</param>
        Protected Overrides Sub AddGlyphsForControl(ByVal c As Control)
            If Not (TypeOf c Is Form) Then
                Me.adornerValue.Glyphs.Add(New PaddingGlyph( _
                    c, _
                    MyBase.BehaviorService, _
                    MyBase.ComponentChangeService, _
                    MyBase.SelectionService, _
                    MyBase.ReferenceService, _
                    MyBase.ServiceProvider, _
                    Me.adornerValue))
            End If
        End Sub

        ''' <summary>
        ''' Removes the <see cref="PaddingGlyph"/> associated with
        ''' the given control from the <see cref="Adorner.Glyphs"/> collection.
        ''' </summary>
        ''' <param name="c">The <see cref="Control"/> associated with 
        ''' the <see cref="PaddingGlyph"/>.</param>
        Protected Overrides Sub RemoveGlyphsForControl(ByVal c As Control)
            If c IsNot Nothing Then
                For i As Integer = 0 To Me.adornerValue.Glyphs.Count - 1
                    Dim g As PaddingGlyph = CType(Me.adornerValue.Glyphs(i), PaddingGlyph)

                    If Object.ReferenceEquals(c, g.RelatedControl) Then
                        Me.adornerValue.Glyphs.Remove(g)
                        Exit For
                    End If
                Next
            End If
        End Sub


        Public Overrides ReadOnly Property ActionList() As DesignerActionList
            Get
                If Me.actionListValue Is Nothing Then
                    Me.actionListValue = New PaddingActionList(Me)
                End If

                Return Me.actionListValue
            End Get
        End Property
    End Class

End Namespace
